<?php

namespace apexl\Io\modules\subscription\entities;

use apexl\Io\includes\Entity;
use apexl\Io\modules\subscription\entities\operators\subscriptionCreditOperator;

class subscriptionCreditEntity extends Entity{


    public function __construct()
    {
        parent::__construct('subscription_credit');
        $this->setOperator(new subscriptionCreditOperator('subscription_credit'));
    }

}