<?php

namespace apexl\Io\modules\subscription;

use apexl\Config\Singleton;
use apexl\Io\includes\System;
use apexl\Io\services\pathUtility;

class createController {
    public function __construct(pathUtility $path, Singleton $config)
    {
        $this->path = $path;
        $this->config = $config;
    }
}