<?php

namespace apexl\Io\modules\subscription\entities;

use apexl\Io\includes\Entity;
use apexl\Io\modules\subscription\entities\operators\subscriptionChangeOperator;

class subscriptionChangeEntity extends Entity{

    public function __construct()
    {
        parent::__construct('subscription_change');
        $this->setOperator(new subscriptionChangeOperator('subscription_change'));
    }
}