<?php

namespace apexl\Io\modules\subscription\entities;

use apexl\Io\includes\Entity;
use apexl\Io\includes\System;
use apexl\Io\modules\subscription\entities\operators\subscriptionPaymentOperator;

class subscriptionPaymentEntity extends Entity{


    public function __construct()
    {
        parent::__construct('subscription_payment');
        $this->setOperator(new subscriptionPaymentOperator('subscription_payment'));
    }

}